@@ -11,7 +11,10 @@ from bycon import *
 
 def main():
 
-    configuration()
+    try:
+        configuration()
+    except Exception:
+        print_text_response(traceback.format_exc(), byc["env"], 302)    
     
 ################################################################################
 
@@ -21,7 +24,7 @@ def configuration():
     r, e = instantiate_response_and_error(byc, "beaconConfigurationResponse")
     response_meta_set_info_defaults(r, byc)
 
-    c_f = path.join( pkg_path, *byc["config"]["default_model_path"], "beaconConfiguration.json")
+    c_f = get_schema_file_path(byc, "beaconConfiguration")
     c = load_yaml_empty_fallback( c_f )
 
     r.update( {"response": c } )
