@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-import cgi, cgitb, sys
+import cgi, sys
 from os import pardir, path
 
 from bycon import *
@@ -16,12 +16,16 @@ podmd"""
 
 def main():
 
-    info()
+    try:
+        info()
+    except Exception:
+        print_text_response(traceback.format_exc(), byc["env"], 302)
     
 ################################################################################
 
 def info():
 
+    initialize_bycon_service(byc)
     r, e = instantiate_response_and_error(byc, "beaconInfoResponse")
     response_meta_set_info_defaults(r, byc)
 
@@ -37,13 +41,13 @@ def info():
     r["meta"].pop("returned_granularity", None)
 
     pgx_info = b_e_d.get("info", {})
-    info = object_instance_from_schema_name(byc, "beaconInfoResults", "properties", "json")
+    beacon_info = object_instance_from_schema_name(byc, "beaconInfoResults", "")
 
-    for k in info.keys():
+    for k in beacon_info.keys():
         if k in pgx_info:
-            info.update({k:pgx_info[k]})
+            beacon_info.update({k:pgx_info[k]})
 
-    r.update( {"response": info } )
+    r.update( {"response": beacon_info } )
     byc.update({"service_response": r, "error_response": e })
 
     cgi_print_response( byc, 200 )
