@@ -1,10 +1,32 @@
 import re, yaml
 from pymongo import MongoClient
+from pathlib import Path
 from os import environ, pardir, path
 import sys
 
 from cgi_parsing import *
-from export_file_generation import de_vrsify_variant
+from bycon_helpers import hex_2_rgb
+from variant_mapping import ByconVariant
+
+################################################################################
+
+def _handover_add_stringified_plot_parameters(h_o_t, byc):
+
+    if not "plot" in h_o_t:
+        return ""
+
+    p_p = get_plot_parameters({}, byc)
+    p_p_l = []
+    for ppk, ppv in p_p.items():
+        if type(ppv) is list:
+            p_p_l.append( f'{ppk}={ ",".join(map(str, (ppv))) }' )
+            continue
+        p_p_l.append( f'{ppk}={ppv}' )
+
+    if len(p_p_l) < 1:
+        return ""
+
+    return f'&{"&".join(p_p_l)}'
 
 ################################################################################
 
@@ -42,20 +64,18 @@ def dataset_response_add_handovers(ds_id, byc):
                 if "remove_subdomain" in h_o_types[ h_o_t ]:
                     this_server = re.sub(r'\/\/\w+?\.(\w+?\.\w+?)$', r'//\1', this_server)
                 h_o_r = {
-                    "handover_type": {
-                        "id": h_o_defs[ "id" ],
-                        "label": "{}".format(h_o_defs[ "label" ]),
-                    },
-                    "description": h_o_defs[ "description" ],
+                    "handover_type": h_o_defs.get("handoverType", {}),
+                    "info": { "content_id": h_o_t},
+                    "note": h_o_defs[ "note" ],
                     "url": "",
                     "pages": []
                 }
 
-                if "bedfile" in h_o_defs[ "id" ]:
+                if "UCSClink" in h_o_t:
                     bed_file_name, ucsc_pos = _write_variants_bedfile(h_o, 0, 0, byc)
                     h_o_r.update( { "url": _handover_create_ext_url(this_server, h_o_defs, bed_file_name, ucsc_pos, byc ) } )
                 else:
-                    h_o_r.update( { "url": _handover_create_url(this_server, h_o_defs, accessid, byc) } )
+                    h_o_r.update( { "url": handover_create_url(this_server, h_o_defs, accessid, byc) } )
 
                 # TODO: needs a new schema to accommodate this not as HACK ...
                 # the phenopackets URL needs matched variants, which it wouldn't know about ...
@@ -78,19 +98,19 @@ def dataset_response_add_handovers(ds_id, byc):
                         # no re-pagination of the results retrieved from the paginated query
                         # TODO: the bedfile part is wrong, since it paginates by the number of variants which
                         # may be incorrect if biosamples ... were called. have to change...
-                        if "bedfile" in h_o_defs[ "id" ]:
+                        if "bedfile" in h_o_t:
                             bed_file_name, ucsc_pos = _write_variants_bedfile(h_o, p_f, p_t, byc)
                             u =  _handover_create_ext_url(this_server, h_o_defs, bed_file_name, ucsc_pos, byc )
                         else:
                             u = h_o_r["url"] + "&paginateResults=false&skip={}&limit={}".format(p_s, byc["pagination"]["limit"])
-                        h_o_r["pages"].append( { "handover_type": {"id": h_o_defs[ "id" ], "label": l }, "url": u } )
+                        h_o_r["pages"].append( { "handover_type": {"id": h_o_defs["handoverType"][ "id" ], "label": l }, "url": u } )
                         p_s += 1
                         p_f += byc["pagination"]["limit"]
                         p_t = p_f + byc["pagination"]["limit"]
 
                     h_o_r["url"] += "&skip={}&limit={}".format(byc["pagination"]["skip"], byc["pagination"]["limit"])
-
-                b_h_o.append( h_o_r )
+                if "url" in h_o_r:
+                    b_h_o.append( h_o_r )
 
     return b_h_o
 
@@ -112,8 +132,8 @@ def query_results_save_handovers(byc):
 
 def dataset_results_save_handovers(ds_id, byc):
 
-    ho_client = MongoClient()
-    ho_db = ho_client[ byc["config"]["info_db"] ]
+    ho_client = MongoClient(host=environ.get("BYCON_MONGO_HOST", "localhost"))
+    ho_db = ho_client[ byc["config"]["housekeeping_db"] ]
     ho_coll = ho_db[ byc["config"][ "handover_coll" ] ]
 
     for h_o_k in byc["dataset_results"][ds_id].keys():
@@ -131,7 +151,7 @@ def dataset_results_save_handovers(ds_id, byc):
 
 ################################################################################
 
-def _handover_create_url(h_o_server, h_o_defs, accessid, byc):
+def handover_create_url(h_o_server, h_o_defs, accessid, byc):
 
     if "script_path_web" in h_o_defs:
         server = h_o_server
@@ -142,6 +162,7 @@ def _handover_create_url(h_o_server, h_o_defs, accessid, byc):
             if p in h_o_defs:
                 url += "&{}={}".format(p, h_o_defs[p])
         url += h_o_defs.get("url_opts", "")
+        url += _handover_add_stringified_plot_parameters(h_o_defs.get("output", ""), byc)
 
         return url
 
@@ -151,9 +172,13 @@ def _handover_create_url(h_o_server, h_o_defs, accessid, byc):
 
 def _handover_create_ext_url(h_o_server, h_o_defs, bed_file_name, ucsc_pos, byc):
 
+    local_paths = byc.get("local_paths")
+    if not local_paths:
+        return False
+
     if "ext_url" in h_o_defs:
-        if "bedfile" in h_o_defs["id"]:
-            return("{}&position={}&hgt.customText={}{}/{}".format(h_o_defs["ext_url"], ucsc_pos, h_o_server, byc["config"].get("server_tmp_dir_web", "/tmp"), bed_file_name))
+        if "bedfile" in h_o_defs["handoverType"]["id"]:
+            return("{}&position={}&hgt.customText={}{}/{}".format(h_o_defs["ext_url"], ucsc_pos, h_o_server, local_paths.get("server_tmp_dir_web", "/tmp"), bed_file_name))
 
     return False
 
@@ -182,7 +207,12 @@ def _write_variants_bedfile(h_o, p_f, p_t, byc):
 
     podmd"""
 
-    config = byc["config"]
+    local_paths = byc.get("local_paths")
+    if not local_paths:
+        return False
+    tmp_path = Path( path.join( *local_paths[ "server_tmp_dir_loc" ]) )
+    if not tmp_path.is_dir():
+        return False
 
     v_ret = 0
     v_max = 1000
@@ -200,12 +230,12 @@ def _write_variants_bedfile(h_o, p_f, p_t, byc):
     else:
         p_t = v_max # only for the non-paginated ...
 
-    bed_file_name = accessid + l + '.bed'
-    bed_file = path.join( *config[ "server_tmp_dir_loc" ], bed_file_name )
+    bed_file_name = f'{accessid}{l}.bed'
+    bed_file = Path( path.join( tmp_path, bed_file_name ) )
 
     vs = { "DUP": [ ], "DEL": [ ], "LOH": [ ], "SNV": [ ]}
 
-    data_client = MongoClient( )
+    data_client = MongoClient(host=environ.get("BYCON_MONGO_HOST", "localhost"))
     data_db = data_client[ ds_id ]
     data_coll = data_db[ h_o["target_collection"] ]
 
@@ -219,25 +249,22 @@ def _write_variants_bedfile(h_o, p_f, p_t, byc):
             if v_ret <= p_f:
                 continue
         
-        v_d = de_vrsify_variant(v, byc)
-        if v_d is False:
+        # TODO: Just make this from the standard variant format
+        bv = ByconVariant(byc, v)
+        pv = bv.byconVariant()
+
+        if "DUP" in pv["variant_type"]:
+            vs["DUP"].append(pv)
+        elif "DEL" in pv["variant_type"]:
+            vs["DEL"].append(pv)
+        elif "LOH" in pv["variant_type"]:
+            vs["LOH"].append(pv)
+        elif "SNV" in pv["variant_type"]:
+            vs["SNV"].append(pv)
+        else:
             continue
 
-        if "variant_type" in v_d:
-            v_d.update({"size": v_d["end"] - v_d["start"] })
-            if "DUP" in v_d["variant_type"]:
-                vs["DUP"].append(v_d)
-            elif "DEL" in v_d["variant_type"]:
-                vs["DEL"].append(v_d)
-            elif "LOH" in v_d["variant_type"]:
-                vs["LOH"].append(v_d)
-            else:
-                continue
-        elif "reference_bases" in v_d:
-            vs["SNV"].append(v_d)
-
     b_f = open( bed_file, 'w' )
-
     pos = set()
 
     ucsc_chr = ""
@@ -245,11 +272,11 @@ def _write_variants_bedfile(h_o, p_f, p_t, byc):
     for vt in vs.keys():
         if len( vs[vt] ) > 0:
             try:
-                vs[vt] = sorted(vs[vt], key=lambda k: k['size'], reverse=True)
+                vs[vt] = sorted(vs[vt], key=lambda k: k['variant_length'], reverse=True)
             except:
                 pass
             col_key = "plot_{}_color".format(vt.lower())
-            col_hex = byc["plot_defaults"].get(col_key, [])
+            col_hex = byc["plot_defaults"].get(col_key, "#666666")
             col_rgb = hex_2_rgb(col_hex)
             b_f.write("track name={} visibility=squish description=\"{} variants matching the query with {} overall returned\" color={},{},{}\n".format(vt, vt, v_ret, col_rgb[0], col_rgb[1], col_rgb[2] ) )
             b_f.write("#chrom\tchromStart\tchromEnd\tbiosampleId\n")
@@ -268,3 +295,5 @@ def _write_variants_bedfile(h_o, p_f, p_t, byc):
 
     return [bed_file_name, ucsc_pos]
 
+################################################################################
+
