@@ -1,20 +1,23 @@
-import re
-from bson.objectid import ObjectId
+import bson.objectid
 
 from cgi_parsing import *
 from query_execution import mongo_result_list
 
 ################################################################################
 
-def parse_variant_parameters(byc):
+def parse_variants(byc):
+    __parse_variant_parameters(byc)
+    __get_variant_request_type(byc)
 
-    if not "variant_definitions" in byc:
-        return byc
+
+################################################################################
+
+def __parse_variant_parameters(byc):
+    v_d = byc["variant_parameters"]
+    v_p_defs = v_d["parameters"]
+    v_t_defs = byc["variant_type_definitions"]
 
     variant_pars = { }
-    v_p_defs = byc["variant_definitions"]["parameters"]
-    v_t_als = byc["variant_definitions"]["variant_state_aliases"]
-    v_t_defs = byc["variant_definitions"]["variant_states"]
 
     for p_k in v_p_defs.keys():
         v_default = None
@@ -36,12 +39,12 @@ def parse_variant_parameters(byc):
             continue
         v_p = variant_pars[ p_k ]
         if "variant_type" in p_k:
-            if v_p in v_t_als:
-                v_t_k = v_t_als[v_p]
-                if "LiteralSequenceExpression" in v_t_k:
-                    v_p_c[ p_k ] = None
-                else:
-                    v_p_c[ p_k ] = { "$in": v_t_defs[v_t_k]["child_terms"] }
+            v_s = variant_state_from_variant_par(v_p, byc)
+            if v_s is False:
+                v_p_c[ p_k ] = None
+            else:
+                v_s_id = v_s["id"]  # on purpose here leading to error if ill defined
+                v_p_c[ p_k ] = { "$in": v_t_defs[v_s_id]["child_terms"] }
         elif "array" in v_p_defs[ p_k ]["type"]:
             v_l = set()
             for v in v_p:
@@ -58,29 +61,10 @@ def parse_variant_parameters(byc):
 
     byc.update( { "variant_pars": v_p_c } )
 
-    return byc
 
 ################################################################################
 
-def translate_reference_name(variant_pars, byc):
-
-    if not "reference_name" in variant_pars:
-        return variant_pars
-
-    r_n = variant_pars[ "reference_name" ]
-    r_a = byc["variant_definitions"]["refseq_aliases"]
-
-    if not r_n in r_a.keys():
-        variant_pars.pop("reference_name")
-        return variant_pars
-
-    variant_pars.update({"reference_name": r_a[r_n] })
-
-    return variant_pars
-
-################################################################################
-
-def get_variant_request_type(byc):
+def __get_variant_request_type(byc):
 
     """podmd
     This method guesses the type of variant request, based on the complete
@@ -92,14 +76,14 @@ def get_variant_request_type(byc):
     podmd"""
 
     if not "variant_pars" in byc:
-        return byc
+        return
 
     variant_request_type = "no correct variant request"
 
     v_pars = byc["variant_pars"]
-    v_p_defs = byc["variant_definitions"]["parameters"]
+    v_p_defs = byc["variant_parameters"]["parameters"]
 
-    brts = byc["variant_definitions"]["request_types"]
+    brts = byc["variant_parameters"]["request_types"]
     brts_k = brts.keys()
     
     # DODO HACK: setting to range request if start and end with one value
@@ -140,14 +124,78 @@ def get_variant_request_type(byc):
 
     byc.update( { "variant_request_type": variant_request_type } )
 
-    return byc
+
+################################################################################
+
+def variant_state_from_variant_par(variant_type, byc):
+    v_d = byc["variant_parameters"]
+    v_t_defs = byc["variant_type_definitions"]
+
+    for k, d in v_t_defs.items():
+        for p, v in d.items():
+            if v is None:
+                continue
+            if type(v) is list:
+                continue
+            if "variant_state" in p:
+                v = v.get("id", "___none___")
+            if type(v) is not str:
+                continue
+            if variant_type.lower() == v.lower():
+                return d["variant_state"]
+
+    return False
+
+
+################################################################################
+
+def variant_vcf_type_from_variant_par(variant_type, byc):
+    v_d = byc["variant_parameters"]
+    v_t_defs = byc["variant_type_definitions"]
+
+    for k, d in v_t_defs.items():
+        for p, v in d.items():
+            if v is None:
+                continue
+            if type(v) is list:
+                continue
+            if "variant_state" in p:
+                v = v.get("id", "___none___")
+            if type(v) is not str:
+                continue
+            if variant_type.lower() == v.lower():
+                return d.get("VCF")
+
+    return False
+
+
+################################################################################
+
+def translate_reference_name(variant_pars, byc):
+
+    if not "reference_name" in variant_pars:
+        return variant_pars
+
+    r_n = variant_pars[ "reference_name" ]
+    g_a = byc.get("genome_aliases", {})
+    r_a = g_a.get("refseq_aliases", {})
+
+    if not r_n in r_a.keys():
+        variant_pars.pop("reference_name")
+        return variant_pars
+
+    variant_pars.update({"reference_name": r_a[r_n] })
+
+    return variant_pars
+
+
 
 ################################################################################
 
 def create_variantTypeRequest_query( byc ):
 
     if byc["variant_request_type"] != "variantTypeRequest":
-        return byc
+        return
 
     """
     A query just for a variant type w/o any additional parameter has to be blocked
@@ -157,10 +205,10 @@ def create_variantTypeRequest_query( byc ):
     """
 
     if len(byc.get("filters", [])) < 1:
-        return byc
+        return
 
     vp = byc["variant_pars"]
-    v_p_defs = byc["variant_definitions"]["parameters"]
+    v_p_defs = byc["variant_parameters"]["parameters"]
 
     v_q_l = [
         { v_p_defs["variant_type"]["db_key"]: vp[ "variant_type" ] }
@@ -176,45 +224,41 @@ def create_variantTypeRequest_query( byc ):
 
     expand_variant_query(v_q, byc)
 
-    return byc
-
 ################################################################################
 
 def create_variantIdRequest_query( byc ):
 
     if byc["variant_request_type"] != "variantIdRequest":
-        return byc
+        return
 
     # query database for gene and use coordinates to create range query
     vp = byc["variant_pars"]
-    v_p_defs = byc["variant_definitions"]["parameters"]
+    v_p_defs = byc["variant_parameters"]["parameters"]
 
     if "_id" in vp:
-        v_q = { v_p_defs["_id"]["db_key"] : ObjectId( vp[ "_id" ] ) }
+        v_q = {v_p_defs["_id"]["db_key"] : bson.objectid.ObjectId(vp["_id"])}
     elif "id" in vp:
         v_q = { 
             "$or": [
-                { v_p_defs["_id"]["db_key"] : ObjectId( vp[ "id" ] ) },
+                {v_p_defs["_id"]["db_key"] : bson.objectid.ObjectId(vp["id"])},
                 { v_p_defs["id"]["db_key"] : vp[ "id" ] }
             ]
         }
     else:
-        return byc
+        return
 
     expand_variant_query(v_q, byc)
 
-    return byc
-
 ################################################################################
 
 def create_geneVariantRequest_query( byc ):
 
     if byc["variant_request_type"] != "geneVariantRequest":
-        return byc
+        return
 
     # query database for gene and use coordinates to create range query
     vp = byc["variant_pars"]
-    v_p_defs = byc["variant_definitions"]["parameters"]
+    v_p_defs = byc["variant_parameters"]["parameters"]
 
 
     query = { "symbol" : vp[ "gene_id" ] }
@@ -232,8 +276,6 @@ def create_geneVariantRequest_query( byc ):
     byc.update( {"variant_request_type": "variantRangeRequest"} )
     create_variantRangeRequest_query( byc )
 
-    return byc
-
 ################################################################################
 
 def create_variantAlleleRequest_query( byc ):
@@ -243,10 +285,10 @@ def create_variantAlleleRequest_query( byc ):
     podmd"""
 
     if byc["variant_request_type"] != "variantAlleleRequest":
-        return byc
+        return
 
     vp = byc["variant_pars"]
-    v_p_defs = byc["variant_definitions"]["parameters"]
+    v_p_defs = byc["variant_parameters"]["parameters"]
 
     # TODO: Regexes for ref or alt with wildcard characters
 
@@ -266,17 +308,15 @@ def create_variantAlleleRequest_query( byc ):
 
     expand_variant_query(v_q, byc)
 
-    return byc
-
 ################################################################################
 
 def create_variantCNVrequest_query( byc ):
 
     if not byc["variant_request_type"] in [ "variantCNVrequest" ]:
-        return byc
+        return
 
     vp = byc["variant_pars"]
-    v_p_defs = byc["variant_definitions"]["parameters"]
+    v_p_defs = byc["variant_parameters"]["parameters"]
 
     v_q = { "$and": [
         { v_p_defs["reference_name"]["db_key"]: vp[ "reference_name" ] },
@@ -289,17 +329,15 @@ def create_variantCNVrequest_query( byc ):
 
     expand_variant_query(v_q, byc)
 
-    return byc
-
 ################################################################################
 
 def create_variantRangeRequest_query( byc ):
 
     if not byc["variant_request_type"] in [ "variantRangeRequest" ]:
-        return byc
+        return
     
     vp = byc["variant_pars"]
-    v_p_defs = byc["variant_definitions"]["parameters"]
+    v_p_defs = byc["variant_parameters"]["parameters"]
 
     v_q_l = [
         { v_p_defs["reference_name"]["db_key"]: vp[ "reference_name" ] },
@@ -328,8 +366,6 @@ def create_variantRangeRequest_query( byc ):
 
     expand_variant_query(v_q, byc)
 
-    return byc
-
 ################################################################################
 
 def expand_variant_query(variant_query, byc):
@@ -339,8 +375,6 @@ def expand_variant_query(variant_query, byc):
     else:
         byc["queries"].update( {"variants": variant_query } )
 
-    return byc
-
 ################################################################################
 
 def create_and_or_query_for_list(logic, q_list):
@@ -377,17 +411,13 @@ def create_in_query_for_parameter(par, qpar, q_pars):
 
 def variant_create_digest(v, byc):
 
-    v_d = byc["variant_definitions"]
+    # TODO: remove / part of ByconVariant
 
     t = v["variant_state"]["id"]
     t = re.sub(":", "_", t)
 
-    v_i = v["location"]["interval"]
-    p = "{}-{}".format(v_i["start"]["value"], v_i["end"]["value"])
-
-    chro = chroname_from_refseqid(v["location"]["sequence_id"], byc)
-
-    return ":".join( [chro, p, t])
+    v_i = v["location"]
+    return f'{v_i["chromosome"]}:{v_i["start"]}-{v_i["end"]}:{t}'
 
 ################################################################################
 
@@ -399,3 +429,62 @@ def chroname_from_refseqid(refseqid, byc):
     return chro
 
 ################################################################################
+
+def normalize_pgx_variant(variant, byc, counter=1):
+    g_a = byc.get("genome_aliases", {})
+    r_a = g_a.get("refseq_aliases", {})
+    c_a = g_a.get("chro_aliases", {})
+    v_t_defs = byc["variant_type_definitions"]
+    errors = []
+
+    var_id = variant.get("id", counter)
+
+    seq_id = variant["location"].get("sequence_id")
+    chromosome = variant["location"].get("chromosome")
+    start = variant["location"].get("start")
+    end = variant["location"].get("end")
+    if not seq_id:
+        if chromosome:
+            variant["location"].update({"sequence_id": r_a.get(str(chromosome))})
+    if not chromosome:
+        if seq_id:
+            variant["location"].update({"chromosome": c_a.get(str(seq_id))})
+    if not isinstance(end, int):
+        try:
+            ref = variant.get("reference_sequence")
+            alt = variant.get("sequence")
+            v_l = len(ref) - len(alt)
+            end_pos = start + abs(v_l) + 1
+            # TODO: VRS would do a left-clipping -> start shift ...
+            variant["location"].update({"end": end_pos})
+        except:
+            pass
+
+    # TODO: Some fixes ...
+    if "-" in variant.get("sequence", "."):
+        variant["sequence"] = re.sub("-", "", variant["sequence"])
+    if "-" in variant.get("reference_sequence", "."):
+        variant["sequence"] = re.sub("-", "", variant["reference_sequence"])
+
+    var_state_id = variant["variant_state"].get("id")
+    variant_type = variant.get("variant_type")
+    if not var_state_id:
+        if variant_type:
+            variant.update({ "variant_state": variant_state_from_variant_par(variant_type, byc) })
+
+    try:
+        variant["variant_state"].update({"label": v_t_defs[var_state_id].get("label")})
+    except:
+        pass
+
+
+    for v_l_k in [ "sequence_id", "chromosome", "start", "end" ]:
+        if not variant["location"].get(v_l_k):
+            errors.append(f'¡¡¡ Parameter `location.{v_l_k}` undefined in variant {var_id} !!!')
+    for v_s_k in [ "id", "label" ]:
+        if not variant["variant_state"].get(v_s_k):
+            errors.append(f'¡¡¡ Parameter `variant_state.{v_s_k}` undefined in variant {var_id} !!!')
+
+    return variant, errors
+
+################################################################################
